﻿using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using ThinkGeo.MapSuite.Core;

namespace Sample
{
    internal class Program
    {
        private static void Main(string[] args)
        {
            string sourcePathFileName = @"..\..\..\..\Sample\Data\iringa_region.shp";
            string targetPathFileName = @"..\..\..\..\Sample\Sample\bin\Debug\result.shp";

            DeleteFiles(targetPathFileName);

            ShapeFileFeatureLayer sourceLayer = new ShapeFileFeatureLayer(sourcePathFileName);
            sourceLayer.Open();

            Collection<Feature> features = sourceLayer.QueryTools.GetAllFeatures(ReturningColumnsType.AllColumns);
            IEnumerable<IGrouping<string, Feature>> groupFeatures = features.GroupBy(f => f.ColumnValues["ward_name"]);
            Collection<Feature> unionFeatures = new Collection<Feature>();
            foreach (var item in groupFeatures)
            {
                // we don't have any exceptions in this process, if you still have the exception in the post, please uncomment the code.
                MultipolygonShape multipolygon = MultipolygonShape.Union(item);
                //MultipolygonShape multipolygon = MultipolygonShape.Union(item.Select(f => SqlTypesGeometryHelper.MakeValid(f)));

                foreach (var polygon in multipolygon.Polygons)
                {
                    //// option 1, we can clear all the inner rings inside of the polygon.
                    //polygon.InnerRings.Clear();

                    // option 2, we can optionally remove the rings inside of the polygon.
                    for (int i = polygon.InnerRings.Count - 1; i >= 0; i--)
                    {
                        double area = polygon.InnerRings[i].GetArea(GeographyUnit.DecimalDegree, AreaUnit.SquareMeters);
                        if (area < 20) polygon.InnerRings.RemoveAt(i);
                    }
                }

                // we can decide how to retain the unioned column values.
                Feature newFeature = new Feature(multipolygon, item.First().ColumnValues);
                unionFeatures.Add(newFeature);
            }

            IEnumerable<DbfColumn> targetColumns = sourceLayer.FeatureSource.GetColumns()
                .Select(c => new DbfColumn(c.ColumnName, DbfColumnType.Character, c.MaxLength, 0));
            ShapeFileFeatureLayer.CreateShapeFile(ShapeFileType.Polygon, targetPathFileName, targetColumns);
            ShapeFileFeatureLayer targetLayer = new ShapeFileFeatureLayer(targetPathFileName, ShapeFileReadWriteMode.ReadWrite);
            targetLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.State1;
            targetLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.Capital3("dist_name");

            // this property can fix your missing label issue.
            targetLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.DuplicateRule = LabelDuplicateRule.UnlimitedDuplicateLabels;
            targetLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            targetLayer.Open();

            targetLayer.EditTools.BeginTransaction();
            foreach (var feature in unionFeatures) targetLayer.EditTools.Add(feature);
            targetLayer.EditTools.CommitTransaction();

            Bitmap bitmap = new Bitmap(1024, 1024);
            GdiPlusGeoCanvas canvas = new GdiPlusGeoCanvas();
            canvas.BeginDrawing(bitmap, targetLayer.GetBoundingBox(), GeographyUnit.DecimalDegree);
            targetLayer.Draw(canvas, new Collection<SimpleCandidate>());
            canvas.EndDrawing();

            string imagePath = Path.ChangeExtension(targetPathFileName, ".png");
            bitmap.Save(imagePath);
            Process.Start(imagePath);
        }

        private static void DeleteFiles(string targetPath)
        {
            string[] array = { ".shp", ".shx", ".ids", ".idx", ".dbf", ".prj", ".png" };
            foreach (var item in array)
            {
                string tempFile = Path.ChangeExtension(targetPath, item);
                if (File.Exists(tempFile))
                {
                    File.Delete(tempFile);
                }
            }
        }
    }
}